﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Microsoft.Win32;  // Registry
using System.Windows;
using System.Runtime.InteropServices;
using System.ComponentModel;  // MessageBox


namespace FM6000_Listview {
    class Basis {

        [DllImport("kernel32.dll", SetLastError = true, PreserveSig = true)]
        static extern int GetDiskFreeSpaceW([In, MarshalAs(UnmanagedType.LPWStr)] string lpRootPathName,
           out uint lpSectorsPerCluster, out uint lpBytesPerSector, out uint lpNumberOfFreeClusters,
           out uint lpTotalNumberOfClusters);

        public static long GetCluster(string root) {
            //FileInfo info = new FileInfo(file);
            uint dummy, sectorsPerCluster, bytesPerSector;
            int result = GetDiskFreeSpaceW(root, out sectorsPerCluster, out bytesPerSector, out dummy, out dummy);
            if (result == 0) throw new Win32Exception();
            return sectorsPerCluster * bytesPerSector;
        }

        // path = Application.StartupPath;  // nur WinForms
        //path = Directory.GetCurrentDirectory(); //System.Environment.CurrentDirectory;
        // path = System.Environment.CurrentDirectory;            
        public static String getProgPath() {
            string[] args = System.Environment.GetCommandLineArgs();
            String exepath = args[0];
            int i = exepath.LastIndexOf("\\");
            String path = exepath.Substring(0, i + 1);
            if (!path.EndsWith("\\")) path += "\\";
            return path;

        }

        public static MessageBoxResult readYesNoCancel(string message, string title) {
            return MessageBox.Show(message, title, MessageBoxButton.YesNoCancel, MessageBoxImage.Question);
        }

        public static bool readYesNo(string message, string title) {
            return MessageBox.Show(message, title, MessageBoxButton.YesNo, MessageBoxImage.Question) == MessageBoxResult.Yes;
        }

        public static void ErrorMsg(String text, String title) {
            MessageBox.Show(text, title, MessageBoxButton.OK, MessageBoxImage.Error);
        }

        public static String Long_Str_Komma(long number) {
            int i;
            String zeile = number.ToString();
            i = zeile.Length;

            if (number >= 1000) zeile = zeile.Insert(i - 3, ",");
            if (number >= 1000000) zeile = zeile.Insert(i - 6, ",");
            if (number >= 1000000000) zeile = zeile.Insert(i - 9, ",");
            return zeile;
        }  // Long_Str_Komma

        public static String Best_KB(long size) {
          double s;          
          s = size / 1024.0;
          if (size < 1000) {              
              return size + " Bytes";
          }
          else {
              if (s < 1000) {
                  return s.ToString("#.#") + " KB";
              }
              else {
                  // MB oder GB                  
                  s = s / 1024.0;
                  if (s < 1000.0) {
                      return s.ToString("#.#") + " MB";
                  }
                  else {
                      s = s / 1024.0;
                      return s.ToString("#.##") + " GB";
                  }
              }
          }
        }


        public static bool CheckDateinamen(String filename) {
            if (filename.Length == 0)
                return false;
            else {
                bool result = true;
                for (int i = 0; i < filename.Length; i++) {
                    String ch = filename.Substring(i, 1);
                    switch (ch) {
                        case "\\": result = false; break;
                        case "/": result = false; break;
                        case ":": result = false; break;
                        case "?": result = false; break;
                        case "*": result = false; break;
                        case "\"": result = false; break;
                        case "<": result = false; break;
                        case ">": result = false; break;
                        case "|": result = false; break;
                    }// case
                }// for
                return result;
            }// if
        }  //  pruefe_dateinamen 

        public static bool CheckDateinamenWithPath(String filename) {
            if (filename.Length == 0)
                return false;
            else {
                bool result = true;
                for (int i = 0; i < filename.Length; i++) {
                    String ch = filename.Substring(i, 1);
                    switch (ch) {
                        case "/": result = false; break;
                        case "?": result = false; break;
                        case "*": result = false; break;
                        case "\"": result = false; break;
                        case "<": result = false; break;
                        case ">": result = false; break;
                        case "|": result = false; break;
                    }// case
                }// for
                return result;
            }// if
        }  //  pruefe_dateinamen 


        public static void splitFilename(String pathFilename, out String path, out String filename) {
            int i = pathFilename.LastIndexOf("\\");
            if (i >= 0) {
                path = pathFilename.Substring(0, i + 1);
                filename = pathFilename.Substring(i + 1);
                filename = pathFilename.Substring(i + 1);
            }
            else {
                filename = pathFilename;
                path = "";
            }
        }

        public static String HeaderFolder(String path) {
            if (path.Length <= 3) {
                return path;
            }
            else {
                if (path.EndsWith("\\")) {
                    path = path.Substring(0, path.Length - 1);
                }
                int i = path.LastIndexOf("\\");
                if (i >= 0) {
                    string[] splitStrings = { "\\" };
                    String[] items = path.Split(splitStrings, StringSplitOptions.None);
                    if (items.Length <= 2)
                        return path;
                    else
                        return path.Substring(0, 3) + "..." + path.Substring(i + 1);
                }
                else {
                    return path;
                }
            }
        }

        public static String getLastFolder(String path) {
            if (path.Length <= 3) {
                return path;
            }
            else {
                if (path.EndsWith("\\")) {
                    path = path.Substring(0, path.Length - 1);
                }
                int i = path.LastIndexOf("\\");
                if (i >= 0) {
                    return path.Substring(i + 1);
                }
                else {
                    return path;
                }
            }
        }

        public static String getPath(String filename) {
            int i = filename.LastIndexOf("\\");
            if (i >= 0) {
                return filename.Substring(0,i + 1);
            }
            else {
                return filename;
            }
        }

        /// <summary>
        /// return only the extention, sample: test.TXT return txt
        /// </summary>
        /// <param name="sPathFilename"></param>
        /// <returns></returns>
        public static String getFilenameExt(String sPathFilename) {
            int k = sPathFilename.LastIndexOf('.');
            if (k >= 0) {
                return sPathFilename.Substring(k + 1).ToLower();
            }
            else {
                return "";
            }
        }  // getFilename


        /// <returns></returns>
        public static void getPathNameExt(String pathfilename, out String path, out String name, out String ext) {
            int k = pathfilename.LastIndexOf('\\');
            if (k >= 0) {
                path = pathfilename.Substring(0, k);
            }
            else {
                path = "";
            }
            String filename = pathfilename.Substring(k + 1);
            k = filename.LastIndexOf('.');
            if (k >= 0) {
                name = filename.Substring(0, k);
                ext = filename.Substring(k + 1);
            }
            else {
                name = filename;
                ext = "";
            }
        }  // getFilename


        /// <summary>
        /// return only the name and the extention, test.txt
        /// </summary>
        /// <param name="sPathFilename"></param>
        /// <returns></returns>
        public static String getFilename(String sPathFilename) {
            int k = sPathFilename.LastIndexOf('\\');
            if (k >= 0) {
                return sPathFilename.Substring(k + 1);
            }
            else {
                return sPathFilename;
            }
        }  // getFilename


        public static String ReadRegString(string root, string KeyName) {
            string sKey;
            RegistryKey regist;
            try {
                regist = Registry.CurrentUser.OpenSubKey(root);
                if (regist == null) {
                    //MessageBox.Show("Fehler", "Registry-Eintrag ist nicht vorhanden: " + KeyName.ToUpper());
                    return "";
                }
                else {
                    if (regist.GetValue(KeyName) == null)
                        sKey = "";
                    else {
                        sKey = regist.GetValue(KeyName).ToString();
                    }
                    return sKey;
                }
            }
            catch (Exception e) {
                // OOOOH, an Error
                Basis.ErrorMsg(e.ToString(), "Writing registry " + KeyName.ToUpper());
                return "";
            }
        }  // readRegString


        public static long ReadRegLong(string root, string KeyName) {
            long lKey;
            string sKey;
            RegistryKey regist;
            try {
                regist = Registry.CurrentUser.OpenSubKey(root);
                if (regist == null) {
                    //MessageBox.Show("Fehler", "Registry-Eintrag ist nicht vorhanden: " + KeyName.ToUpper());
                    return 0;
                }
                else {
                    if (regist.GetValue(KeyName) == null)
                        lKey = 0;
                    else {
                        sKey = regist.GetValue(KeyName).ToString();
                        lKey = (long)Convert.ToUInt64(sKey);
                    }
                    return lKey;
                }
            }
            catch (Exception e) {
                // OOOOH, an Error
                ErrorMsg(e.ToString(), "Reading registry long" + KeyName.ToUpper());
                return 0;
            }
        }  // readRegLong


        public static int ReadRegInt(string root, string KeyName) {
            int lKey;
            string sKey;
            RegistryKey regist;
            try {
                regist = Registry.CurrentUser.OpenSubKey(root);
                if (regist == null) {
                    //MessageBox.Show("Fehler", "Registry-Eintrag ist nicht vorhanden: " + KeyName.ToUpper());
                    return 0;
                }
                else {
                    if (regist.GetValue(KeyName) == null)
                        lKey = 0;
                    else {
                        sKey = regist.GetValue(KeyName).ToString();
                        lKey = (int)Convert.ToInt32(sKey);
                        //.ToUInt64(sKey);
                    }
                    return lKey;
                }
            }
            catch (Exception e) {
                // OOOOH, an Error
                ErrorMsg(e.ToString(), "Reading registry int" + KeyName.ToUpper());
                return 0;
            }
        }  // readRegInt

        public static ulong ReadRegULong(string root, string KeyName) {
            string sKey;
            ulong lKey = 0;
            RegistryKey regist;
            try {
                regist = Registry.CurrentUser.OpenSubKey(root);
                if (regist == null) {
                    //MessageBox.Show("Fehler", "Registry-Eintrag ist nicht vorhanden: " + KeyName.ToUpper());
                    return 0;
                }
                else {
                    if (regist.GetValue(KeyName) == null)
                        lKey = 0;
                    else {
                        sKey = regist.GetValue(KeyName).ToString();
                        lKey = Convert.ToUInt64(sKey);
                    }
                    return lKey;
                }
            }
            catch (Exception e) {
                // OOOOH, an Error
                ErrorMsg(e.ToString(), "Reading registry double" + KeyName.ToUpper());
                return 0;
            }
        }  // readRegLong


        public static double ReadRegDouble(string root, string KeyName) {
            string sKey;
            double dKey = 0;
            RegistryKey regist;
            try {
                regist = Registry.CurrentUser.OpenSubKey(root);
                if (regist == null) {
                    //MessageBox.Show("Fehler", "Registry-Eintrag ist nicht vorhanden: " + KeyName.ToUpper());
                    return 0.0;
                }
                else {
                    if (regist.GetValue(KeyName) == null)
                        dKey = 0.0;
                    else {
                        sKey = regist.GetValue(KeyName).ToString();
                        dKey = Convert.ToDouble(sKey);
                    }
                    return dKey;
                }
            }
            catch (Exception e) {
                // OOOOH, an Error
                ErrorMsg(e.ToString(), "Reading registry double" + KeyName.ToUpper());
                return 0.0;
            }
        }  // readRegDouble



        // WriteWriteWriteWriteWriteWriteWriteWriteWriteWriteWriteWrite


        public static bool WriteRegString(string root, string KeyName, string Value) {
            try {
                RegistryKey rKey = Registry.CurrentUser.CreateSubKey(root);
                if (rKey == null) {
                    //MessageBox.Show("Fehler", "Registry-Eintrag ist nicht vorhanden: " + KeyName.ToUpper());
                    return false;
                }
                else {
                    rKey.SetValue(KeyName, Value, RegistryValueKind.String);
                    return true;
                }
            }
            catch (Exception e) {
                // OOOOH, an Error
                ErrorMsg(e.ToString(), "Writing registry string" + KeyName.ToUpper());
                return false;
            }
        }



        public static bool WriteRegULong(string root, string KeyName, ulong Value) {
            try {
                RegistryKey rKey = Registry.CurrentUser.CreateSubKey(root);
                if (rKey == null) {
                    //MessageBox.Show("Fehler", "Registry-Eintrag ist nicht vorhanden: " + KeyName.ToUpper());
                    return false;
                }
                else {
                    rKey.SetValue(KeyName, Value, RegistryValueKind.DWord);
                    return true;
                }
            }
            catch (Exception e) {
                // OOOOH, an Error
                ErrorMsg(e.ToString(), "Writing registry long" + KeyName.ToUpper());
                return false;
            }
        }

        // fuer long int etc
        public static bool WriteRegInt(string root, string KeyName, int Value) {
            try {
                RegistryKey rKey = Registry.CurrentUser.CreateSubKey(root);
                if (rKey == null) {
                    //MessageBox.Show("Fehler", "Registry-Eintrag ist nicht vorhanden: " + KeyName.ToUpper());
                    return false;
                }
                else {
                    rKey.SetValue(KeyName, Value, RegistryValueKind.DWord);
                    return true;
                }
            }
            catch (Exception e) {
                // OOOOH, an Error
                ErrorMsg(e.ToString(), "Writing registry int" + KeyName.ToUpper());
                return false;
            }
        }



        public static bool WriteRegDouble(string root, string KeyName, double Value) {
            try {
                RegistryKey rKey = Registry.CurrentUser.CreateSubKey(root);
                if (rKey == null) {
                    //MessageBox.Show("Fehler", "Registry-Eintrag ist nicht vorhanden: " + KeyName.ToUpper());
                    return false;
                }
                else {
                    rKey.SetValue(KeyName, Value.ToString(), RegistryValueKind.String);
                    return true;
                }
            }
            catch (Exception e) {
                // OOOOH, an Error
                ErrorMsg(e.ToString(), "Writing registry double" + KeyName.ToUpper());
                return false;
            }
        }

        public static bool ReadRegBool(string root, string KeyName) {
            bool bKey;
            RegistryKey regist;
            try {
                regist = Registry.CurrentUser.OpenSubKey(root);
                if (regist == null) {
                    //MessageBox.Show("Fehler", "Registry-Eintrag ist nicht vorhanden: " + KeyName.ToUpper());
                    return false;
                }
                else {
                    if (regist.GetValue(KeyName) == null)
                        bKey = false;
                    else {
                        int value = (int)regist.GetValue(KeyName);
                        bKey = value == 1;
                    }
                    return bKey;
                }
            }
            catch (Exception e) {
                // OOOOH, an Error
                ErrorMsg(e.ToString(), "Reading registry bool" + KeyName.ToUpper());
                return false;
            }
        }  // readRegBool

        public static bool WriteRegBool(string root, string KeyName, bool Value) {
            try {
                RegistryKey rKey = Registry.CurrentUser.CreateSubKey(root);
                if (rKey == null) {
                    //MessageBox.Show("Fehler", "Registry-Eintrag ist nicht vorhanden: " + KeyName.ToUpper());
                    return false;
                }
                else {
                    if (Value)
                        rKey.SetValue(KeyName, 1, RegistryValueKind.DWord);  // Bool geht nicht                    
                    else
                        rKey.SetValue(KeyName, 0, RegistryValueKind.DWord);  // Bool geht nicht                    
                    return true;
                }
            }
            catch (Exception e) {
                // OOOOH, an Error
                ErrorMsg(e.ToString(), "Writing registry bool" + KeyName.ToUpper());
                return false;
            }
        }



    }
}
