﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;


using System.IO;
using System.Diagnostics;
using Microsoft.Win32;

namespace FM6000_Listview {
    /// <summary>
    /// Interaction logic for FEditor.xaml
    /// </summary>
    public partial class FEditor : Window {


        private String _actualDirectory = "";

        private String _content;

        private int _fontsize = 12;


        public FEditor(String content) {
            InitializeComponent();
            this._content = content;
        }

        private void Window_Loaded(object sender, RoutedEventArgs e) {
            setGUI();
            _actualDirectory = Environment.CurrentDirectory;
            _fontsize = 12;
            mnFont12pg.IsChecked = true;
            editor._editor.Text = _content;
        }


        private void setGUI() {
            CommandBindings.Add(new CommandBinding(MyEditorCommands.NewFile, NewFileExecuted));
            CommandBindings.Add(new CommandBinding(MyEditorCommands.OpenFile, OpenFileExecuted));
            CommandBindings.Add(new CommandBinding(MyEditorCommands.SaveFile, SaveFileExecuted));
            CommandBindings.Add(new CommandBinding(MyEditorCommands.SaveAsFile, SaveAsFileExecuted));

            CommandBindings.Add(new CommandBinding(MyEditorCommands.Undo, UndoExecuted));
            CommandBindings.Add(new CommandBinding(MyEditorCommands.Redo, RedoExecuted));
            CommandBindings.Add(new CommandBinding(MyEditorCommands.Copy, CopyExecuted));
            CommandBindings.Add(new CommandBinding(MyEditorCommands.Paste, PasteExecuted));
            CommandBindings.Add(new CommandBinding(MyEditorCommands.Cut, CutExecuted));
            CommandBindings.Add(new CommandBinding(MyEditorCommands.Delete, DeleteExecuted));
            CommandBindings.Add(new CommandBinding(MyEditorCommands.SelectAll, SelectAllExecuted));

            CommandBindings.Add(new CommandBinding(MyEditorCommands.Wordwrap, WordwrapExecuted));

            CommandBindings.Add(new CommandBinding(MyEditorCommands.Close, CloseExecuted));


            CommandBindings.Add(new CommandBinding(MyEditorCommands.DosEbene, DosEbeneExecuted));

            CommandBindings.Add(new CommandBinding(MyEditorCommands.Options, OptionsExecuted));
            CommandBindings.Add(new CommandBinding(MyEditorCommands.ProgInfo, ProgInfoExecuted));

        }


        private void loadFile(String filename) {
            editor.loadFile(filename);
        }



        private void CloseExecuted(object sender, ExecutedRoutedEventArgs e) {
            Close();
        }



        private void NewFileExecuted(object sender, ExecutedRoutedEventArgs e) {
            editor.NewFile();
        }

        private void OpenFileExecuted(object sender, ExecutedRoutedEventArgs e) {
            OpenFileDialog openFileDialog1 = new OpenFileDialog();
            openFileDialog1.Filter = "Text-Dateien (*.txt,*.dat)|*.txt;*.dat|" +
                                     "Programm-Dateien|*.java;*.cs;*.pas;*.php|" +
                                     "HTML-Dateien|*.htm*;*.xhtml;*.php;*.js|" +
                                     "XML-Dateien (*.xml)|*.xml|Alle Dateien (*.*)|*.*";
            openFileDialog1.FilterIndex = 1;// zählt von 1 !
            openFileDialog1.DefaultExt = ".txt";
            openFileDialog1.InitialDirectory = _actualDirectory;
            openFileDialog1.Multiselect = false;
            openFileDialog1.RestoreDirectory = true;
            if (openFileDialog1.ShowDialog() == true) {
                _actualDirectory = Basis.getPath(openFileDialog1.FileName.ToString());
                editor.loadFile(openFileDialog1.FileName.ToString());
            }
        }

        private void SaveFileExecuted(object sender, ExecutedRoutedEventArgs e) {
            if (editor.Filename.Length > 0)
                editor.SaveFile();
            else
                SaveAsFileExecuted(sender, e);
        }

        private void SaveAsFileExecuted(object sender, ExecutedRoutedEventArgs e) {
            SaveFileDialog saveFileDialog1 = new SaveFileDialog();
            saveFileDialog1.Filter = "Text-Dateien (*.txt,*.dat)|*.txt;*.dat|" +
                                     "Programm-Dateien|*.java;*.cs;*.pas;*.php|" +
                                     "HTML-Dateien|*.htm*;*.xhtml;*.php;*.js|" +
                                     "XML-Dateien (*.xml)|*.xml|Alle Dateien (*.*)|*.*";
            saveFileDialog1.FilterIndex = 1;// zählt von 1 !
            saveFileDialog1.DefaultExt = ".txt";
            saveFileDialog1.InitialDirectory = _actualDirectory;
            saveFileDialog1.RestoreDirectory = true;
            if (saveFileDialog1.ShowDialog() == true) {
                _actualDirectory = Basis.getPath(saveFileDialog1.FileName.ToString());
                editor.Filename = saveFileDialog1.FileName.ToString();
                editor.SaveFile();
            }
        }

        private void UndoExecuted(object sender, ExecutedRoutedEventArgs e) {
            editor.Undo();
        }
        private void RedoExecuted(object sender, ExecutedRoutedEventArgs e) {
            editor.Redo();
        }

        private void DeleteExecuted(object sender, ExecutedRoutedEventArgs e) {
            editor.Delete();
        }

        private void CopyExecuted(object sender, ExecutedRoutedEventArgs e) {
            editor.Copy();
        }
        private void PasteExecuted(object sender, ExecutedRoutedEventArgs e) {
            editor.Paste();
        }
        private void CutExecuted(object sender, ExecutedRoutedEventArgs e) {
            editor.Cut();
        }

        private void SelectAllExecuted(object sender, ExecutedRoutedEventArgs e) {
            editor.SelectAll();
        }



        private void WordwrapExecuted(object sender, ExecutedRoutedEventArgs e) {
            mnWordWrap.IsChecked = !mnWordWrap.IsChecked;
            editor.WordWrap = mnWordWrap.IsChecked;
        }




        private void clearFontMenues(object sender) {
            mnFont8pg.IsChecked = false;
            mnFont10pg.IsChecked = false;
            mnFont12pg.IsChecked = false;
            mnFont14pg.IsChecked = false;
            mnFont16pg.IsChecked = false;
            mnFont18pg.IsChecked = false;
            mnFont20pg.IsChecked = false;
            mnFont24pg.IsChecked = false;
            mnFont28pg.IsChecked = false;
            MenuItem mnItem = (MenuItem)sender;
            mnItem.IsChecked = true;
        }

        private void mnFont_Click(object sender, RoutedEventArgs e) {
            MenuItem mn = (MenuItem)sender;
            clearFontMenues(sender);
            setfontsize(Convert.ToInt32(mn.Tag));
        }

        private void setfontsize(int size) {
            this._fontsize = size;
            editor.setFontsize(size);
            mainMenu.FontSize = size;
        }

        private void mnExplorer_Click(object sender, RoutedEventArgs e) {
            Process Prog = new Process();
            Prog.StartInfo.FileName = "Explorer.exe";
            Prog.StartInfo.Arguments = "";
            Prog.Start();
        }

        private void mnNotepad_Click(object sender, RoutedEventArgs e) {
            Process Prog = new Process();
            Prog.StartInfo.FileName = "Notepad.exe";
            Prog.StartInfo.Arguments = "";
            Prog.Start();
        }

        private void DosEbeneExecuted(object sender, ExecutedRoutedEventArgs e) {
            Process Prog = new Process();
            Prog.StartInfo.FileName = "CMD.exe";
            Prog.StartInfo.Arguments = "";
            Prog.StartInfo.WorkingDirectory = _actualDirectory;
            Prog.Start();
        }

        private void mnTaskManager_Click(object sender, RoutedEventArgs e) {
            Process Prog = new Process();
            Prog.StartInfo.FileName = "Taskmgr.exe";
            Prog.StartInfo.Arguments = "";
            Prog.Start();
        }

        private void mnSystemsteuerung_Click(object sender, RoutedEventArgs e) {
            Process Prog = new Process();
            Prog.StartInfo.FileName = "control.exe";
            Prog.StartInfo.Arguments = "";
            Prog.Start();
        }

        private void mnSytemConfig_Click(object sender, RoutedEventArgs e) {
            Process Prog = new Process();
            Prog.StartInfo.FileName = "msconfig.exe";
            Prog.StartInfo.Arguments = "";
            Prog.Start();
        }

        private void mnmsinfo32_Click(object sender, RoutedEventArgs e) {
            Process Prog = new Process();
            Prog.StartInfo.FileName = "msinfo32.exe";
            Prog.StartInfo.Arguments = "";
            Prog.Start();
        }



        private void mnRegedit_Click(object sender, RoutedEventArgs e) {
            Process Prog = new Process();
            Prog.StartInfo.FileName = "Regedit.exe";
            Prog.StartInfo.Arguments = "";
            Prog.Start();
        }

        private void mnCalc_Click(object sender, RoutedEventArgs e) {
            Process Prog = new Process();
            Prog.StartInfo.FileName = "Calc.exe";
            Prog.StartInfo.Arguments = "";
            Prog.Start();
        }


        private void OptionsExecuted(object sender, ExecutedRoutedEventArgs e) {
            FOptions dialog = new FOptions(this);
            dialog.ShowDialog();
            if (dialog.Retcode) {
                setFonts();
            }
        }

        private void setFonts() {
            mainMenu.FontSize = MyRegistry.MenuSize;
            // editor.FontSize = MyRegistry.TreeSize;           
        }


        private void ProgInfoExecuted(object sender, ExecutedRoutedEventArgs e) {
            FProgInfo dialog = new FProgInfo(this);
            dialog.ShowDialog();
        }



    }
}
