﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;


using System.IO;
using System.Collections;
using System.Data;

namespace FM6000_Listview {
    /// <summary>
    /// Interaction logic for FOptions.xaml
    /// </summary>
    public partial class FOptions : Window {

        DataTable dtF = new DataTable();
        DataTable dtP = new DataTable();
        DataTable dtL = new DataTable();

        private ArrayList _listeFavoriten = new ArrayList();
        private ArrayList _listeProgs = new ArrayList();
        private ArrayList _listeLinks = new ArrayList();
        private bool retcode = false;

        public bool Retcode {
            get { return retcode; }
        }

        public FOptions(Window parent) {
            InitializeComponent();
            this.Owner = parent;  // Important, because CenterOwner  
            this.WindowStartupLocation = WindowStartupLocation.CenterOwner;          
        }

        private void Window_Loaded(object sender, RoutedEventArgs e) {
            setGUI();
        }

        private void setGUI() {
            cbFontSizeMenue.SelectedIndex = MyRegistry.MenuSize - 8;
            cbFontSizeTree.SelectedIndex = MyRegistry.TreeSize - 8;
            cbFontSizeListView.SelectedIndex = MyRegistry.ListViewSize - 8;
            String path = Basis.getProgPath();
            setTable(gridF, dtF);
            setTable(gridP, dtP);
            setTable(gridL, dtL);
            Middleware.readTexte(path, Konstanten.C_FAVORITEN, true, ref _listeFavoriten); 
            insertListeInGrid(_listeFavoriten, gridF, dtF);
            Middleware.readTexte(path, Konstanten.C_PROGS, false, ref _listeProgs);
            insertListeInGrid(_listeProgs, gridP, dtP);
            Middleware.readTexte(path, Konstanten.C_LINKS, false, ref _listeLinks);
            insertListeInGrid(_listeLinks, gridL, dtL);
        }


        private void setTable(DataGrid grid, DataTable dt) {
            grid.IsReadOnly = true;

            //Columns erstellen
            DataColumn column;

            column = new DataColumn();
            column.DataType = Type.GetType("System.String");
            column.ColumnName = "Bezeichnung";
            dt.Columns.Add(column);


            column = new DataColumn();
            column.DataType = Type.GetType("System.String");

            column.ColumnName = "Pfad";
            dt.Columns.Add(column);

            //DataGrid füllen
            grid.ItemsSource = dt.DefaultView;

            grid.CanUserAddRows = false;
            grid.CanUserDeleteRows = false;
            grid.CanUserResizeColumns = true;
            grid.CanUserSortColumns = false;
        }

        private void insertListeInGrid(ArrayList liste, DataGrid grid, DataTable dt) {
            DataRow row;
            dt.Clear();
            foreach (Favorit f in liste) {
                row = dt.NewRow();
                row["Bezeichnung"] = f.Bez;
                row["Pfad"] = f.Path;
                dt.Rows.Add(row);
            }
        }


        private void bnOk_Click(object sender, RoutedEventArgs e) {
            MyRegistry.MenuSize = cbFontSizeMenue.SelectedIndex + 8;
            MyRegistry.TreeSize = cbFontSizeTree.SelectedIndex + 8;
            MyRegistry.ListViewSize = cbFontSizeListView.SelectedIndex + 8;

            String path = Basis.getProgPath();
            FileStream oFile = new FileStream(path + Konstanten.C_FAVORITEN, FileMode.Create);
            StreamWriter oStream = new StreamWriter(oFile);
            foreach (Favorit f in _listeFavoriten) {
                oStream.WriteLine(f.Bez);
                oStream.WriteLine(f.Path);
            }
            oStream.Close();
            oFile.Close();

            oFile = new FileStream(path + Konstanten.C_PROGS, FileMode.Create);
            oStream = new StreamWriter(oFile);
            foreach (Favorit f in _listeProgs) {
                oStream.WriteLine(f.Bez);
                oStream.WriteLine(f.Path);
            }
            oStream.Close();
            oFile.Close();
            retcode = true;
            Close();


            retcode = true;
            Close();
        }

        private void bnEsc_Click(object sender, RoutedEventArgs e) {
            Close();
        }

        private void Window_KeyDown(object sender, KeyEventArgs e) {
            if (Keyboard.IsKeyDown(Key.Return)) {
                bnOk_Click(sender, e);
            }
            if (Keyboard.IsKeyDown(Key.Escape)) {
                bnEsc_Click(sender, e);
            }
        }

        private void bnNewF_Click(object sender, RoutedEventArgs e) {
            FReadFavorit dialog = new FReadFavorit(this, "", "");
            dialog.ShowDialog();
            if (dialog.Retcode) {
                Favorit f = new Favorit();
                f.Bez = dialog.Bez;
                f.Path = dialog.Path;
                _listeFavoriten.Add(f);
                insertListeInGrid(_listeFavoriten, gridF, dtF);
            }
        }

        private void EditF_Click(int row) {
            if (row >= 0 && row < _listeFavoriten.Count) {
                Favorit f = (Favorit)_listeFavoriten[row];
                FReadFavorit dialog = new FReadFavorit(this, f.Bez, f.Path);
                dialog.ShowDialog();
                if (dialog.Retcode) {
                    f.Bez = dialog.Bez;
                    f.Path = dialog.Path;
                    insertListeInGrid(_listeFavoriten, gridF, dtF);
                }
            }
        }

        private void bnEditF_Click(object sender, RoutedEventArgs e) {
            int row = gridF.SelectedIndex;
            if (row >= 0 && row < _listeFavoriten.Count) {
                EditF_Click(row);
            }
        }
        private void gridF_PreviewMouseDoubleClick(object sender, MouseButtonEventArgs e) {
            bnEditF_Click(sender, null);
        }

        private Favorit getSelectedFavorit(DataGrid grid) {
            if (grid.SelectedItem != null) {
                Favorit f = (Favorit)grid.SelectedItem;
                if (f != null) {
                    return f;
                }
            }
            return null;
        }

        private void bnDeleteF_Click(object sender, RoutedEventArgs e) {
            // find row for the first selected item
            int row = gridF.SelectedIndex;
            if (row >= 0 && row < _listeFavoriten.Count) {
                Favorit f = (Favorit)_listeFavoriten[row];
                if (Basis.readYesNo("Wollen Sie wirklich den Pfad löschen?\r\n" +
                    "Bez: " + f.Bez + "\r\n" +
                    "Pfad: " + f.Path + "\r\n", "Eintrag löschen")) {
                    _listeFavoriten.RemoveAt(row);
                    insertListeInGrid(_listeFavoriten, gridF, dtF);
                }
            }
        }  // bnDeleteF_Click

        private void bnTopF_Click(object sender, RoutedEventArgs e) {
            int row = gridF.SelectedIndex;
            if (row > 0 && row < _listeFavoriten.Count) {
                Favorit f = (Favorit)_listeFavoriten[row];
                _listeFavoriten.RemoveAt(row);
                _listeFavoriten.Insert(row - 1, f);
                insertListeInGrid(_listeFavoriten, gridF, dtF);
                gridF.SelectedIndex = row - 1;
            }

        }

        private void bnDownF_Click(object sender, RoutedEventArgs e) {
            int row = gridF.SelectedIndex;
            if (row >= 0 && row < (_listeFavoriten.Count - 1)) {
                Favorit f = (Favorit)_listeFavoriten[row];
                _listeFavoriten.RemoveAt(row);
                _listeFavoriten.Insert(row + 1, f);
                insertListeInGrid(_listeFavoriten, gridF, dtF);
                gridF.SelectedIndex = row + 1;
            }
        }

        // ------------------------

        private void bnNewP_Click(object sender, RoutedEventArgs e) {
            FReadProg dialog = new FReadProg(this, "", "");
            dialog.ShowDialog();
            if (dialog.Retcode) {
                Favorit f = new Favorit();
                f.Bez = dialog.Bez;
                f.Path = dialog.Prog;
                _listeProgs.Add(f);
                insertListeInGrid(_listeProgs, gridP, dtP);
            }
        }

        private void EditP_Click(int row) {
            if (row >= 0 && row < _listeProgs.Count) {
                Favorit f = (Favorit)_listeProgs[row];
                FReadProg dialog = new FReadProg(this, f.Bez, f.Path);
                dialog.ShowDialog();
                if (dialog.Retcode) {
                    f.Bez = dialog.Bez;
                    f.Path = dialog.Prog;
                    insertListeInGrid(_listeProgs, gridP, dtP);
                }
            }
        }

        private void bnEditP_Click(object sender, RoutedEventArgs e) {
            int row = gridP.SelectedIndex;
            if (row >= 0 && row < (_listeProgs.Count - 1)) {
                EditP_Click(row);
            }

        }
        private void gridP_PreviewMouseDoubleClick(object sender, MouseButtonEventArgs e) {
            bnEditP_Click(sender, null);
        }

        private void bnDeleteP_Click(object sender, RoutedEventArgs e) {
            // find row for the first selected item
            int row = gridP.SelectedIndex;
            if (row >= 0 && row < (_listeProgs.Count - 1)) {
                Favorit f = (Favorit)_listeProgs[row];
                if (Basis.readYesNo("Wollen Sie wirklich das Programm löschen?\r\n" +
                    "Bez: " + f.Bez + "\r\n" +
                    "Programm: " + f.Path + "\r\n", "Eintrag löschen")) {
                    _listeProgs.RemoveAt(row);
                    insertListeInGrid(_listeProgs, gridP, dtP);
                }
            }

        }

        private void bnTopP_Click(object sender, RoutedEventArgs e) {
            int row = gridP.SelectedIndex;
            if (row > 0 && row < _listeProgs.Count) {
                Favorit f = (Favorit)_listeProgs[row];
                _listeProgs.RemoveAt(row);
                _listeProgs.Insert(row - 1, f);
                insertListeInGrid(_listeProgs, gridP, dtP);
                gridP.SelectedIndex = row - 1;
            }
        }

        private void bnDownP_Click(object sender, RoutedEventArgs e) {
            int row = gridP.SelectedIndex;
            if (row >= 0 && row < (_listeProgs.Count - 1)) {
                Favorit f = (Favorit)_listeProgs[row];
                _listeProgs.RemoveAt(row);
                _listeProgs.Insert(row + 1, f);
                insertListeInGrid(_listeProgs, gridP, dtP);
                gridP.SelectedIndex = row + 1;
            }
        }


        // ------------------------------------------------------------------
        private void bnNewL_Click(object sender, RoutedEventArgs e) {
            FReadProg dialog = new FReadProg(this, "", "");
            dialog.ShowDialog();
            if (dialog.Retcode) {
                Favorit f = new Favorit();
                f.Bez = dialog.Bez;
                f.Path = dialog.Prog;
                _listeLinks.Add(f);
                insertListeInGrid(_listeLinks, gridL, dtL);
            }
        }

        private void EditL_Click(int row) {
            if (row >= 0 && row < _listeLinks.Count) {
                Favorit f = (Favorit)_listeLinks[row];
                FReadProg dialog = new FReadProg(this, f.Bez, f.Path);
                dialog.ShowDialog();
                if (dialog.Retcode) {
                    f.Bez = dialog.Bez;
                    f.Path = dialog.Prog;
                    insertListeInGrid(_listeLinks, gridL, dtL);
                }
            }
        }

        private void bnEditL_Click(object sender, RoutedEventArgs e) {
            int row = gridL.SelectedIndex;
            if (row >= 0 && row < (_listeLinks.Count - 1)) {
                EditL_Click(row);
            }

        }
        private void gridL_PreviewMouseDoubleClick(object sender, MouseButtonEventArgs e) {
            bnEditL_Click(sender, null);
        }

        private void bnDeleteL_Click(object sender, RoutedEventArgs e) {
            // find row for the first selected item
            int row = gridL.SelectedIndex;
            if (row >= 0 && row < (_listeLinks.Count - 1)) {
                Favorit f = (Favorit)_listeLinks[row];
                if (Basis.readYesNo("Wollen Sie wirklich die Datei löschen?\r\n" +
                    "Bez: " + f.Bez + "\r\n" +
                    "Programm: " + f.Path + "\r\n", "Eintrag löschen")) {
                    _listeLinks.RemoveAt(row);
                    insertListeInGrid(_listeLinks, gridL, dtL);
                }
            }

        }

        private void bnTopL_Click(object sender, RoutedEventArgs e) {
            int row = gridL.SelectedIndex;
            if (row > 0 && row < _listeLinks.Count) {
                Favorit f = (Favorit)_listeLinks[row];
                _listeLinks.RemoveAt(row);
                _listeLinks.Insert(row - 1, f);
                insertListeInGrid(_listeLinks, gridL, dtL);
                gridL.SelectedIndex = row - 1;
            }
        }

        private void bnDownL_Click(object sender, RoutedEventArgs e) {
            int row = gridL.SelectedIndex;
            if (row >= 0 && row < (_listeLinks.Count - 1)) {
                Favorit f = (Favorit)_listeLinks[row];
                _listeLinks.RemoveAt(row);
                _listeLinks.Insert(row + 1, f);
                insertListeInGrid(_listeLinks, gridL, dtL);
                gridL.SelectedIndex = row + 1;
            }
        }


    }
}
