﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

using System.IO;
using System.Runtime.InteropServices;
using System.ComponentModel;
using System.Diagnostics;
using System.Security.Principal;


namespace FM6000_Listview {
    /// <summary>
    /// Interaction logic for FProperty.xaml
    /// </summary>
    public partial class FProperty : Window {

        private String _filename;
        private bool _modify = false;
        private bool isFolder = false;

        private static   DateTime creationTime= DateTime.Now;
        private static DateTime lastWriteTime = DateTime.Now;
        private static DateTime lastAccessTime = DateTime.Now;


        public FProperty(String filename) {
            InitializeComponent();
            _filename = filename;
        }

        private void Window_Loaded(object sender, RoutedEventArgs e) {
            Title = "Eigenschaften von " + Basis.getFilename(_filename);
            title2.Content = Title;
            setGUI1();
            setGUI2();
            bnUebernehmen.IsEnabled = false;
            bnOk.IsEnabled = false;  // checkbox Click enabled the buttons
        }


        private void setGUI1() {
            FileInfo fileItem = new FileInfo(_filename);
            FileAttributes attribs = fileItem.Attributes;
            DriveInfo drv = new DriveInfo(_filename.Substring(0, 3));
            long clustersize = Basis.GetCluster(_filename.Substring(0, 3));
            if ((attribs & FileAttributes.Directory) == FileAttributes.Directory) {
                lFiletyp2.Content = "Verzeichnis";
                lOpenWith2.Content = "-";
                lFolder2.Content = "-";
                lSize2.Content = "-";
                lSizeCluster2.Content = "-";
                bnGetFolderFileSize.Visibility = Visibility.Visible;
                isFolder = true;
            }
            else {
                lFiletyp2.Content = "Datei";
                lOpenWith2.Content = "";
                lFolder2.Content = Basis.getPath(_filename);
                lSize2.Content = Basis.Best_KB(fileItem.Length) + "  (" + fileItem.Length + ") Bytes";
                long size = fileItem.Length / clustersize;
                if (fileItem.Length % clustersize > 0) size++;
                size *= clustersize;
                lSizeCluster2.Content = Basis.Best_KB(size) + "  (" + size + ") Bytes";
                bnGetFolderFileSize.Visibility = Visibility.Collapsed;
            }
            lCreateTime2.Content = fileItem.CreationTime;
            lChangeTime2.Content = fileItem.LastWriteTime;
            lLastAccess2.Content = fileItem.LastAccessTime;

            chkReadonly.IsChecked = fileItem.IsReadOnly;
            chkHidden.IsChecked = (attribs & FileAttributes.Hidden) == FileAttributes.Hidden;
            chkArchiv.IsChecked = (attribs & FileAttributes.Archive) == FileAttributes.Archive;
            chkSystem.IsChecked = (attribs & FileAttributes.System) == FileAttributes.System;
            chkCompressed.IsChecked = (attribs & FileAttributes.Compressed) == FileAttributes.Compressed;
            chkEncrypted.IsChecked = (attribs & FileAttributes.Encrypted) == FileAttributes.Encrypted;

        }  // setGUI1

        private void setGUI2() {
            GridView gridView = new GridView();
            GridViewColumn col;
            col = new GridViewColumn();
            col.Header = "Eigenschaft";
            col.DisplayMemberBinding = new Binding("Property");
            col.Width = 140;
            gridView.Columns.Add(col);

            col = new GridViewColumn();
            col.Header = "Wert";
            col.DisplayMemberBinding = new Binding("Value");
            col.Width = 220;
            gridView.Columns.Add(col);
            fileListe.View = gridView;

            if (isFolder) return;

            FileVersionInfo fvi = FileVersionInfo.GetVersionInfo(_filename);
            fileListe.Items.Add(new VersionInfo("Firma", fvi.CompanyName));
            fileListe.Items.Add(new VersionInfo("Copyright", fvi.LegalCopyright));
            fileListe.Items.Add(new VersionInfo("Trademarks", fvi.LegalTrademarks));

            fileListe.Items.Add(new VersionInfo("Build-Nr", fvi.FileMajorPart.ToString() + "." + fvi.FileMinorPart.ToString()));
            fileListe.Items.Add(new VersionInfo("Einzelteilnr", fvi.FilePrivatePart));
            fileListe.Items.Add(new VersionInfo("Version", fvi.FileVersion));
            fileListe.Items.Add(new VersionInfo("Interner Name", fvi.InternalName));
            fileListe.Items.Add(new VersionInfo("Original-Dateiname", fvi.OriginalFilename));

            fileListe.Items.Add(new VersionInfo("Product-Build", fvi.ProductBuildPart));
            fileListe.Items.Add(new VersionInfo("Product-Name", fvi.ProductName));
            fileListe.Items.Add(new VersionInfo("ProductVersion", fvi.ProductVersion));
            fileListe.Items.Add(new VersionInfo("Sprache", fvi.Language));

            //fileListe.Items.Add(new VersionInfo("Private-Build", fvi.PrivateBuild));
            //fileListe.Items.Add(new VersionInfo("Special-Build", fvi.SpecialBuild));

            fileListe.Items.Add(new VersionInfo("Kommentar", fvi.Comments));
            fileListe.Items.Add(new VersionInfo("Dateibeschreibung", fvi.FileDescription));
        }




        private void chkReadonly_Click(object sender, RoutedEventArgs e) {
            _modify = true;
            bnUebernehmen.IsEnabled = true;
            bnOk.IsEnabled = true;
        }

        private void chkHidden_Click(object sender, RoutedEventArgs e) {
            _modify = true;
            bnUebernehmen.IsEnabled = true;
            bnOk.IsEnabled = true;
        }

        private void chkArchiv_Click(object sender, RoutedEventArgs e) {
            _modify = true;
            bnUebernehmen.IsEnabled = true;
            bnOk.IsEnabled = true;
        }

        private void chkSystem_Click(object sender, RoutedEventArgs e) {
            _modify = true;
            bnUebernehmen.IsEnabled = true;
            bnOk.IsEnabled = true;
        }


        class VersionInfo {
            String _property;
            String _value;
            public VersionInfo(String property, String value) {
                Property = property;
                if (value != null)
                    Value = value;
                else
                    Value = "-";
            }
            public VersionInfo(String property, int value) {
                Property = property;
                Value = value.ToString();
            }


            public String Property {
                get { return _property; }
                set { _property = value; }
            }
            public String Value {
                get { return _value; }
                set { _value = value; }
            }

        }
        private void bnGetFolderFileSize_Click(object sender, RoutedEventArgs e) {
            bnGetFolderFileSize.Visibility = Visibility.Collapsed;
            long filecount;
            long filesize;
            long clusterfilesize;
            createFilesRekursiv(_filename, out filecount, out filesize, out clusterfilesize);
            lSize1.Content = "Anzahl der Dateien";
            lSize2.Content = filecount.ToString();
            lSizeCluster1.Content = "Summe in Bytes";
            lSizeCluster2.Content = Basis.Best_KB(filesize) + "  (mit Clustergröße:" + Basis.Best_KB(clusterfilesize) + ")";
        }

        private void createFilesRekursiv(String dirFilename, out long filecount, out long filesize, out long clusterfilesize) {
            String[] files = Directory.GetFiles(dirFilename, "*.*", SearchOption.AllDirectories);
            filecount = files.Length;
            filesize = 0;
            clusterfilesize = 0;
            long size;
            long clustersize = Basis.GetCluster(dirFilename.Substring(0, 3));
            foreach (String filename in files) {
                FileInfo fi = new FileInfo(filename);
                filesize += fi.Length;
                size = fi.Length / clustersize;
                if (fi.Length % clustersize > 0) size++;
                clusterfilesize += size * clustersize;
            }
        }

        private void Window_KeyDown(object sender, KeyEventArgs e) {
            if (Keyboard.IsKeyDown(Key.Return)) {
                bnOk_Click(sender, e);
            }
            if (Keyboard.IsKeyDown(Key.Escape)) {
                bnEsc_Click(sender, e);
            }
        }

        private void bnEsc_Click(object sender, RoutedEventArgs e) {
            Close();
        }

        private void bnOk_Click(object sender, RoutedEventArgs e) {
            Save();
            Close();
        }
        private void bnUebernehmen_Click(object sender, RoutedEventArgs e) {
            Save();
        }

        private void setAttrib(FileAttributes fileattrib) {
            File.SetAttributes(_filename, File.GetAttributes(_filename) | fileattrib);
        }
        private void deleteAttrib(FileAttributes fileattrib) {
            try {

                File.SetAttributes(_filename, File.GetAttributes(_filename) & ~fileattrib);
            }
            catch (Exception e) {
                Basis.ErrorMsg("Der Zugriff auf die Datei wurde verweigert", "Hinweis");
            }

        }

        public static bool IsAdministrator() {
            WindowsIdentity identity = WindowsIdentity.GetCurrent();

            if (identity != null) {
                WindowsPrincipal principal = new WindowsPrincipal(identity);
                return principal.IsInRole(WindowsBuiltInRole.Administrator);
            }

            return false;
        }

        private void Save() {
            FileInfo fileItem = new FileInfo(_filename);
            FileAttributes attribs = fileItem.Attributes;
            
            bool bitReadonly;
            if (chkReadonly.IsChecked == true)
                bitReadonly = true;
            else
                bitReadonly = false;
            if (bitReadonly != ((attribs & FileAttributes.ReadOnly) == FileAttributes.ReadOnly)) {
                if (bitReadonly)
                    setAttrib(FileAttributes.ReadOnly);
                else
                    deleteAttrib(FileAttributes.ReadOnly);
            }

            bool bitHidden;
            if (chkHidden.IsChecked == true)
                bitHidden = true;
            else
                bitHidden = false;
            if (bitHidden != ((attribs & FileAttributes.Hidden) == FileAttributes.Hidden)) {
                if (bitHidden)
                    setAttrib(FileAttributes.Hidden);
                else
                    deleteAttrib(FileAttributes.Hidden);
            }

            bool bitArchiv;
            if (chkArchiv.IsChecked == true)
                bitArchiv = true;
            else
                bitArchiv = false;
            if (bitArchiv != ((attribs & FileAttributes.Archive) == FileAttributes.Archive)) {
                if (bitArchiv)
                    setAttrib(FileAttributes.Archive);
                else
                    deleteAttrib(FileAttributes.Archive);
            }

            bool bitSystem;
            if (chkSystem.IsChecked == true)
                bitSystem = true;
            else
                bitSystem = false;
            if (bitSystem != ((attribs & FileAttributes.System) == FileAttributes.System)) {
                if (bitSystem)
                    setAttrib(FileAttributes.System);
                else
                    deleteAttrib(FileAttributes.System);
            }

        }

        private void bnSetTimes_Click(object sender, RoutedEventArgs e) {
            FileInfo fileItem = new FileInfo(_filename);
            fileItem.CreationTime=creationTime;
            fileItem.LastWriteTime=lastWriteTime;
            fileItem.LastAccessTime=lastAccessTime;

        }

        private void bnSaveTimes_Click(object sender, RoutedEventArgs e) {
            FileInfo fileItem = new FileInfo(_filename);
            creationTime= fileItem.CreationTime;
            lastWriteTime = fileItem.LastWriteTime;
            lastAccessTime = fileItem.LastAccessTime;
        }




    }
}
