﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

using System.IO;


namespace FM6000_Listview {
    /// <summary>
    /// Interaction logic for FShowCountRows.xaml
    /// </summary>
    public partial class FShowCountRows : Window {

        private String path;

        public FShowCountRows(Window parent, String path) {
            InitializeComponent();
            this.Owner = parent;  // Important, because CenterOwner            
            this.WindowStartupLocation = WindowStartupLocation.CenterOwner;
            this.path=path;
        }

        private void Window_Loaded(object sender, RoutedEventArgs e) {
            tPath.Text = path;
            tMask.Focus();
        }


        private void bnRun_Click(object sender, RoutedEventArgs e) {
            String mask = tMask.Text.Trim();
            String path = tPath.Text.Trim();
            if (!Directory.Exists(path)) {
                Basis.ErrorMsg("Das Verzeichnis ist nicht vorhanden", "Hinweis");
                tPath.Focus();
                return;
            }
            String noneMask = tNoneMask.Text.Trim();
            String[] noneMasks = null;
            if (noneMask.Length>0) {
                string[] splitStrings = { ";" };
                noneMasks = noneMask.Split(splitStrings, StringSplitOptions.None);
            }

            StringBuilder sb = new StringBuilder();
            Rows rowSummen = calc(path,mask, noneMasks, sb);

            sb.AppendLine("\r\nGesamtergebnis\r\n  Anzahl der Zeilen: " + rowSummen.rows + "\r\n" +
                "  Anzahl der nichtleeren Zeilen: " + rowSummen.rowsNoBlank);
             tOutput.Text = sb.ToString();
        }  // bnRun_Click

        private Rows calc(String path, String maske, String[] nonemask, StringBuilder sb) {
            Rows rowSummen = new Rows();
            DirectoryInfo mainDir = new DirectoryInfo(path);
            FileInfo[] files = mainDir.GetFiles(maske, SearchOption.TopDirectoryOnly);
            foreach (FileInfo file in files) {
                String filenameLower = file.FullName.ToLower();
                bool ok = true;
                if (nonemask != null) {
                    foreach (String mask in nonemask) {
                        if (filenameLower.IndexOf(mask)>=0) {
                            ok=false;
                            break;
                        }
                    }
                }
                if (ok) {
                    String[] items = File.ReadAllLines(file.FullName);
                    foreach (String item in items) {
                        if (item.Trim().Length >0) {
                            rowSummen.rowsNoBlank++;
                        }
                        rowSummen.rows++;
                    }
                }
            }
            if (rowSummen.rows > 0) {
                sb.Append("\r\n");
                sb.AppendLine(path.ToUpper());
                sb.AppendLine("  Anzahl der Zeilen: " + rowSummen.rows + "\r\n" + "  Anzahl der nichtleeren Zeilen: " + rowSummen.rowsNoBlank);
            }

            String[] dirs = Directory.GetDirectories(path);
            foreach (String item in dirs) {
                DirectoryInfo dir = new DirectoryInfo(item);
                Rows summenDirs = calc(dir.FullName, maske, nonemask, sb);
                rowSummen.rowsNoBlank += summenDirs.rowsNoBlank;
                rowSummen.rows += summenDirs.rows;
            }

            return rowSummen;
        }

        private void bnEsc_Click(object sender, RoutedEventArgs e) {
            Close();
        }

        private void Window_KeyDown(object sender, KeyEventArgs e) {
            if (Keyboard.IsKeyDown(Key.Return)) {
                bnRun_Click(sender, e);
            }
            if (Keyboard.IsKeyDown(Key.Escape)) {
                bnEsc_Click(sender, e);
            }
        }

        private class Rows {
            public int rows=0;
            public int rowsNoBlank = 0;
        }


    }
}
