﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using System.IO;
using System.ComponentModel;
using System.Collections.ObjectModel;
using System.Diagnostics; // Sortieren SortDescriptions
using System.Text.RegularExpressions;
using System.Windows.Threading;

namespace FM6000_Listview {
    /// <summary>
    /// Interaktionslogik für MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window, IFileManager, ITabControlManager, IDeleteTabExecuted {

        private DispatcherTimer dispatcherTimer = new DispatcherTimer();

        private TreeViewItem _mainNodeDrives;
        private TreeViewItem _mainNodeFavoriten;

        private TabControl _activeTabControl = null;
        private MyTabItem  _activeTabItem = null;
        private UListView _focusfileTable = null;

        private UListView _fileTableLeft = null;
        private UListView _fileTableRight = null;

        private bool _start = true;

        public MainWindow() {
            InitializeComponent();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e) {
            setGUI();
            int iTop = Basis.ReadRegInt(Konstanten.C_REG_ADR, "iTop1");
            int iLeft = Basis.ReadRegInt(Konstanten.C_REG_ADR, "iLeft1");
            int iWidth = Basis.ReadRegInt(Konstanten.C_REG_ADR, "iWidth1");
            int iHeight = Basis.ReadRegInt(Konstanten.C_REG_ADR, "iHeight1");
            int w = (int)System.Windows.SystemParameters.PrimaryScreenWidth;
            int h = (int)System.Windows.SystemParameters.PrimaryScreenHeight;
            if (iTop > 0 && iLeft > 0 && iLeft < w && iWidth > 50) {
                Top = iTop;
                Left = iLeft;
                Width = iWidth;
                Height = iHeight;
            }
            else {
                Top = 100;
                Left = 20;
            }
            int splitterWidth = Basis.ReadRegInt(Konstanten.C_REG_ADR, "splitterWidth1");
            if (splitterWidth > 50 && splitterWidth < 1000) {
                ColumnDefinitionCollection cols = MainGrid.ColumnDefinitions;
                GridLength gl = new GridLength(splitterWidth);
                cols[0].Width = gl;
            }

            splitterWidth = Basis.ReadRegInt(Konstanten.C_REG_ADR, "splitterWidth2");
            if (splitterWidth > 50 && splitterWidth < 1000) {
                ColumnDefinitionCollection cols = MainGrid.ColumnDefinitions;
                GridLength gl = new GridLength(splitterWidth);
                cols[1].Width = gl;
            }
            Basis.WriteRegInt(Konstanten.C_REG_ADR + "\\FileOp", "AnzFiles", 0);

            _focusfileTable = _fileTableLeft;
            _fileTableLeft.Focus();
            _fileTableLeft.mysetFocus();
            _fileTableRight.mylostFocus();

            dispatcherTimer.Tick += new EventHandler(dispatcherTimer_Tick);
            // dispatcherTimer.Interval = new TimeSpan(0, 0, 1);  // hour, min, sec, ms
            dispatcherTimer.Interval = TimeSpan.FromMilliseconds(2000);

            dispatcherTimer.Start(); 

            String[] files = Directory.GetFiles(Basis.getProgPath(), "kopy*.bat", SearchOption.TopDirectoryOnly);
            foreach(String file in files) {
                File.Delete(file);
            }

            setFonts();
        }

        private void dispatcherTimer_Tick(object sender, EventArgs e) {
            CommandManager.InvalidateRequerySuggested();
            DriveInfo[] drives = DriveInfo.GetDrives();
            if (drives.Length != _mainNodeDrives.Items.Count) {
                insertDrives(_mainNodeDrives);
            }
        }


        private void addNode(TreeViewItem parent, String text, String tag, String pict) {
            StackPanel panel = new StackPanel();
            panel.Orientation = Orientation.Horizontal;

            TextBlock text4 = new TextBlock();
            text4.Text = text;
            Image bild1 = new Image();
            BitmapImage bi1 = new BitmapImage();
            bi1.BeginInit();
            bi1.UriSource = new Uri(pict, UriKind.RelativeOrAbsolute);
            bi1.EndInit();
            bild1.Source = bi1;
            panel.Children.Add(bild1);
            panel.Children.Add(text4);
            TreeViewItem node1 = new TreeViewItem();
            node1.IsTabStop = false;
            node1.Tag = tag;
            node1.Header = panel;
            parent.Items.Add(node1);
        }

        private void setGUI() {
            TreeViewItem root, mainNode, node1; // System.Environment.SpecialFolder
            // tree.Items.Clear();
            // Insert if the roots

            MyRegistry.loadMyRegistry();

            root = itemRoot;
            itemRoot.Tag = null;
            root.IsExpanded = true;
            root.IsTabStop = false;

            // Insert if the roots
            mainNode = itemUser;
            itemUser.Tag = null;

            node1 = new TreeViewItem();
            node1.Tag = Environment.GetFolderPath(Environment.SpecialFolder.CommonPictures);
            node1.Header = "Bilder";
            node1.IsTabStop = false;
            mainNode.Items.Add(node1);

            node1 = new TreeViewItem();
            node1.Tag = Environment.GetFolderPath(Environment.SpecialFolder.DesktopDirectory);
            node1.Header = "Desktop";
            node1.IsTabStop = false;
            mainNode.Items.Add(node1);


            node1 = new TreeViewItem();
            node1.Tag = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
            node1.Header = "Dokumente";
            node1.IsTabStop = false;
            mainNode.Items.Add(node1);

            node1 = new TreeViewItem();
            node1.Tag = Environment.GetFolderPath(Environment.SpecialFolder.DesktopDirectory).Replace("Desktop", "Downloads");
            node1.Header = "Download";
            node1.IsTabStop = false;
            mainNode.Items.Add(node1);

            
            node1 = new TreeViewItem();
            node1.Tag = Environment.GetFolderPath(Environment.SpecialFolder.MyMusic);
            node1.Header = "Musik";
            node1.IsTabStop = false;
            mainNode.Items.Add(node1);

            node1 = new TreeViewItem();
            node1.Tag = System.IO.Path.GetTempPath();
            node1.Header = "Temp-Verz.";
            node1.IsTabStop = false;
            mainNode.Items.Add(node1);

            node1 = new TreeViewItem();
            node1.Tag = Environment.GetFolderPath(Environment.SpecialFolder.MyVideos);
            node1.Header = "Videos";
            node1.IsTabStop = false;
            mainNode.Items.Add(node1);


            node1 = new TreeViewItem();
            node1.Tag = Environment.GetFolderPath(Environment.SpecialFolder.Windows);
            node1.Header = "Windows-Verz.";
            node1.IsTabStop = false;
            mainNode.Items.Add(node1);


            mainNode.IsExpanded = false;

            _mainNodeFavoriten = itemFavoriten; //  new TreeViewItem();
            itemFavoriten.Tag = null;
            insertFavoriten(_mainNodeFavoriten);

            insertProgs(mainProgs);
            mainNode.IsExpanded = false;

            insertLinks(mainLinks);

            _mainNodeDrives = itemDrives; //  new TreeViewItem();
            itemDrives.Tag = null;
            //_mainNodeDrives.Header = "Laufwerke";
            //_mainNodeDrives.Foreground = Brushes.Blue;
            _mainNodeDrives.IsExpanded = true;
            // _mainNodeDrives.Tag = null;
            // root.Items.Add(_mainNodeDrives);
            insertDrives(_mainNodeDrives);

            setFonts();

            CommandBindings.Add(new CommandBinding(MyCommands.Copy, CopyExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.Paste, PasteExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.Cut, CutExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.Delete, DeleteExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.SelectAll, SelectAllExecuted));

            CommandBindings.Add(new CommandBinding(MyCommands.LoadAgain, LoadAgainExecuted));

            CommandBindings.Add(new CommandBinding(MyCommands.CopyF8, CopyF8Executed));
            CommandBindings.Add(new CommandBinding(MyCommands.RenameF2, RenameF2Executed));

            CommandBindings.Add(new CommandBinding(MyCommands.DeleteTab, DeleteTabExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.NewRegister, NewRegisterExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.NewRegisterWithPath, NewRegisterWithPathExecuted));


            CommandBindings.Add(new CommandBinding(MyCommands.GoToTopFolder, GoToTopFolderExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.GoToRoot, GoToRootExecuted));


            CommandBindings.Add(new CommandBinding(MyCommands.NewInstanz, NewInstanzExecuted));

            CommandBindings.Add(new CommandBinding(MyCommands.Close, CloseExecuted));

            CommandBindings.Add(new CommandBinding(MyCommands.Options, OptionsExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.ProgInfo, ProgInfoExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.DosEbene, DosEbeneExecuted));
            

            CommandBindings.Add(new CommandBinding(MyCommands.NewFolder, NewFolderExecuted));

            CommandBindings.Add(new CommandBinding(MyCommands.ShowAllFiles, ShowAllFilesExecuted));

            CommandBindings.Add(new CommandBinding(MyCommands.CompareFolders, CompareFoldersExecuted));

            CommandBindings.Add(new CommandBinding(MyCommands.Path2Clipboard, Path2ClipboardExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.OwnerDefinedMask, OwnerDefinedMaskExecuted));

            CommandBindings.Add(new CommandBinding(MyCommands.RunNotepad, RunNotepadExecuted));

            CommandBindings.Add(new CommandBinding(MyCommands.LoadDrivesAgain, LoadDrivesAgainExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.RunEditor5000, Editor5000Executed));
            CommandBindings.Add(new CommandBinding(MyCommands.RunHexEditor, RunHexEditorExecuted));

            String actualPath1 = Basis.ReadRegString(Konstanten.C_REG_ADR, "Folder1");
            if (!Directory.Exists(actualPath1)) actualPath1 = getFirstFolder();
            MyTabItem tabitem1 = new MyTabItem(this, tabcontrol1);
            tabitem1.IsTabStop = false;
            // MyListView fileTable1 = new MyListView(this, tabcontrol1, tabitem1, actualPath1, im_dir, im_file, new Konstanten.setFilenamePath(setFilenamePath1));
            UListView fileTable1 = new UListView();
            fileTable1.init(this, this, true, tabcontrol1, tabitem1, actualPath1, new DelegateManager.setActiveRegister(setActiveRegister));

            tabitem1.Content = fileTable1;
            tabcontrol1.Items.Add(tabitem1);
            tabitem1.Focus();
             _focusfileTable = fileTable1;

             _activeTabControl = tabcontrol1;
             _activeTabItem = tabitem1;
             _focusfileTable = fileTable1;

             _fileTableLeft = fileTable1;


            String actualPath2 = Basis.ReadRegString(Konstanten.C_REG_ADR, "Folder2");
            if (!Directory.Exists(actualPath2)) actualPath2 = getFirstFolder();
            MyTabItem tabitem2 = new MyTabItem(this, tabcontrol2);
            tabitem2.IsTabStop = false;
            //MyListView fileTable2 = new MyListView(this, tabcontrol2, tabitem2, actualPath2, im_dir, im_file, new Konstanten.setFilenamePath(setFilenamePath2));
            UListView fileTable2 = new UListView();
            fileTable2.init(this, this, false, tabcontrol2, tabitem2, actualPath2, new DelegateManager.setActiveRegister(setActiveRegister));

            tabitem2.Content = fileTable2;
            tabcontrol2.Items.Add(tabitem2);
            int n = tabcontrol2.Items.Count;
            _fileTableRight = fileTable2;

            tabitem2.Focus();
            tabitem1.Focus();
            _activeTabControl = tabcontrol1;
            _start = false;
            
        } // setGUI


        public void activeUListView(UListView listview) {
        }

        public void setActiveRegister(bool left, TabControl tb, MyTabItem tp, UListView lv) {
            //Title = "1";
            _activeTabControl = tb;
            _activeTabItem = tp;
            _focusfileTable = lv;
            if (left) {
                _fileTableLeft = lv;
                // beim create
                if (_fileTableRight!=null) _fileTableRight.mylostFocus();
            }
            else {
                _fileTableRight = lv;
                if (_fileTableLeft!=null) _fileTableLeft.mylostFocus();
            }
        }



        private String getFirstFolder() {
            foreach (TreeViewItem item in _mainNodeDrives.Items) {
                String path = ((String)item.Tag);
                DriveInfo drv = new DriveInfo(path);
                if (drv.IsReady) {
                    return path;
                }
            }
            return "c:\\";
        }




        private void RunNotepadExecuted(object sender, ExecutedRoutedEventArgs e) {
            _focusfileTable.RunNotepad();
        }

        private void LoadDrivesAgainExecuted(object sender, ExecutedRoutedEventArgs e) {
            insertDrives(_mainNodeDrives);
        }

        private void Editor5000Executed(object sender, ExecutedRoutedEventArgs e) {
            _focusfileTable.RunEditor5000();
        }

        private void RunHexEditorExecuted(object sender, ExecutedRoutedEventArgs e) {
            _focusfileTable.RunHexEditor();
        }

        private void CloseExecuted(object sender, ExecutedRoutedEventArgs e) {
            Close();
        }




        private void insertProgs(MenuItem main) {
            MenuItem mnProg;
            String path = Basis.getProgPath();
            String filepath, name;
            main.Items.Clear();
            if (File.Exists(path + Konstanten.C_PROGS)) {
                String[] items = File.ReadAllLines(path + Konstanten.C_PROGS);
                int i = 0;
                while (i < items.Length) {
                    if (items[i].Length > 0) {
                        mnProg = new MenuItem();
                        name = items[i];
                        i++;
                        filepath = items[i];
                        i++;
                        mnProg.Foreground = Brushes.Blue;
                        mnProg.Header = name;
                        mnProg.Tag = filepath;
                        main.Items.Add(mnProg);
                        mnProg.Click += mnProgs_Click;
                    }
                    else {
                        i++;
                        i++;
                    }
                }
            }
        }  // InsertProgs

        private void insertLinks(MenuItem main) {
            MenuItem mnFile;
            String path = Basis.getProgPath();
            String filepath, name;
            main.Items.Clear();
            if (File.Exists(path + Konstanten.C_LINKS)) {
                String[] items = File.ReadAllLines(path + Konstanten.C_LINKS);
                int i = 0;
                while (i < items.Length) {
                    if (items[i].Length > 0) {
                        name = items[i];
                        i++;
                        filepath = items[i];
                        i++;
                        mnFile = new MenuItem();
                        mnFile.Foreground = Brushes.Blue;
                        mnFile.Header = name;
                        mnFile.Tag = filepath;
                        main.Items.Add(mnFile);
                        mnFile.Click += mnProgs_Click;
                    }
                    else {
                        i++;
                        i++;
                    }
                }
            }
        }  // InsertLinks


        private void mnProgs_Click(object sender, RoutedEventArgs e) {
            MenuItem mnProg = (MenuItem)sender;
            String file = (String)mnProg.Tag;
            if (File.Exists(file)) {
                Process Prog = new Process();
                Prog.StartInfo.FileName = "\"" + file + "\"";
                Prog.StartInfo.Arguments = "";
                Prog.Start();
            }
            else {
                Basis.ErrorMsg("Die Datei ist nicht vorhanden\r\n" + file, "Aufrufen eines externen Programms");
            }
        }


        private void insertFavoriten(TreeViewItem parent) {
            // path = Application.StartupPath;  // nur WinForms
            TreeViewItem node;
            String path = Basis.getProgPath();
            parent.Items.Clear();

            //path = Directory.GetCurrentDirectory(); //System.Environment.CurrentDirectory;
            // path = System.Environment.CurrentDirectory;            
            if (File.Exists(path + Konstanten.C_FAVORITEN)) {
                String[] items = File.ReadAllLines(path + Konstanten.C_FAVORITEN);
                int i = 0;
                while (i < items.Length) {
                    if (items[i].Length > 0) {
                        if (!Directory.Exists(items[i + 1])) {
                            Basis.ErrorMsg("Das Verzeichnis ist nicht vorhanden\r\n" + items[i + 1], "Eintragen der Favoriten-Verzeichnisse");
                            i += 2;
                        }
                        else {
                            node = new TreeViewItem();
                            node.Header = items[i];
                            node.IsTabStop = false;
                            i++;
                            node.Tag = items[i];
                            parent.Items.Add(node);
                            i++;
                        }
                    }
                    else {
                        i++;
                        i++;
                    }
                }
            }
        }  // insertFavoriten



        private void insertDrives(TreeViewItem mainNodeDrives) {
            // TreeViewItem node1;
            mainNodeDrives.Items.Clear();
            // String[] drives = System.Environment.GetLogicalDrives();  // get only the Strings
            DriveInfo[] drives = DriveInfo.GetDrives();
            int lfdnr = 0;
            foreach (DriveInfo drv in drives) {
                lfdnr++;
                TreeViewItem node1 = new TreeViewItem();
                node1.IsTabStop = false;
                node1.Tag = drv.RootDirectory.FullName;
                StackPanel panel = new StackPanel();
                panel.Orientation = Orientation.Horizontal;
                // panel.Background = Brushes.Yellow;

                TextBlock text4 = new TextBlock();
                Image bild1 = new Image();

                String typ = "";
                // check, if node1 ist drive c ?
                switch (drv.DriveType) {
                    case DriveType.CDRom: typ = "CD/DVD:"; 
                        bild1.Source = new BitmapImage(
                                new Uri("pack://application:,,,/FM6000_Listview;component/images/cd_dvd.png"));
                                        break;
                    case DriveType.Fixed: typ = "Platte:"; 
                                        bild1.Source = new BitmapImage(
                                                new Uri("pack://application:,,,/FM6000_Listview;component/images/disk.png"));
                                        break;
                    case DriveType.Network: typ = "Netz:"; 
                                        bild1.Source = new BitmapImage(
                                                new Uri("pack://application:,,,/FM6000_Listview;component/images/network.png"));
                                        break;
                    case DriveType.Ram: typ = "Ram:"; break;
                    case DriveType.Removable: typ = "USB:"; 
                        bild1.Source = new BitmapImage(
                                new Uri("pack://application:,,,/FM6000_Listview;component/images/floppy.png"));
                        break;
                    case DriveType.Unknown: typ = "?:"; break;
                }
                if (drv.DriveType != DriveType.CDRom && drv.DriveType != DriveType.Removable && drv.DriveType != DriveType.Network) {
                    // eventuell wird das Laufwerk formatiert
                    try {
                        if (drv.VolumeLabel.Equals("")) {
                            typ = typ + " (" + drv.RootDirectory + ")";
                        }
                        else {
                            typ = drv.VolumeLabel + " (" + drv.RootDirectory + ")";
                        }
                    }
                    catch (Exception e) {
                        typ = typ + " (" + drv.RootDirectory + ")";
                    }
                }
                else {
                    if (drv.DriveType == DriveType.CDRom)
                        typ = typ + " (" + drv.RootDirectory + ")";
                    else
                        typ = typ + " (" + drv.RootDirectory + ")";
                }
                //text4.Text = typ + "  Alt+" + lfdnr.ToString();
                text4.Text = drv.RootDirectory + "  Alt+" + lfdnr.ToString();               
                    
                panel.Children.Add(bild1);
                panel.Children.Add(text4);
                node1.Header = panel;
                mainNodeDrives.Items.Add(node1);
            }  // foreach
        }


        public void insertSubDirsIntoTree(String path, bool clearSubs) {
            TreeViewItem node1 = null;
            TreeViewItem node2, node3;
            if (!Directory.Exists(path)) {
                Basis.ErrorMsg("Das Verzeichnis ist nicht vorhanden\r\n" + path, "Wechseln in ein Verzeichnis");
                return;
            }
            if (path.EndsWith("\\")) path = path.Remove(path.Length - 1);
            String[] dirs1 = path.Split('\\');
            for (int i = 1; i < dirs1.Length; i++) {
                dirs1[i] = dirs1[i - 1] + "\\" + dirs1[i];
            }
            dirs1[0] += "\\";
            _search = true;
            node1 = searchNode(_mainNodeDrives, 0, dirs1);

            bool search = false;
            // zzzzzzzzzzz
            if (clearSubs && node1.HasItems) {
                node1.Items.Clear();
                search = true;
            }
            else {
                // check, if the node hat Items, BUT node with subdirs has at first only ONE node with "?"
                // if the node hat Items, don't search again
                search = !node1.HasItems;
                if (!search) {
                    // test if item ist "?", ween search
                    node2 = (TreeViewItem)node1.Items[0];
                    if (node2.Header.Equals("?")) {
                        search = true;
                        node1.Items.Clear();
                    }
                }
                else {
                }
            }

            if (search) {
                try {
                    // search for subdirs
                    String[] dirs = Directory.GetDirectories(path);
                    foreach (String item in dirs) {
                        DirectoryInfo dir = new DirectoryInfo(item);
                        FileAttributes attribs = dir.Attributes;
                        if ((attribs & FileAttributes.Hidden) != FileAttributes.Hidden && (attribs & FileAttributes.System) != FileAttributes.System) {
                            node2 = new TreeViewItem();
                            node2.IsTabStop = false;


                            StackPanel panel = new StackPanel();
                            panel.Orientation = Orientation.Horizontal;

                            TextBlock text4 = new TextBlock();
                            text4.Text = dir.Name;
                            Image bild1 = new Image();
                            BitmapImage bi1 = new BitmapImage();
                            bi1.BeginInit();
                            bi1.UriSource = new Uri("Directory.png", UriKind.RelativeOrAbsolute);
                            bi1.EndInit();
                            bild1.Source = bi1;
                            double w = bi1.Width;
                            double h = bi1.Height;
                            panel.Children.Add(bild1);
                            panel.Children.Add(text4);
                            node2.Header = panel;
                            node2.Tag = dir.FullName;
                            node1.Items.Add(node2);
                            try {
                                String[] dirs2 = Directory.GetDirectories(dir.FullName);
                                if (dirs2.Length > 0) {
                                    node3 = new TreeViewItem();
                                    node3.IsTabStop = false;
                                    node3.Header = "?";
                                    node3.Tag = null;
                                    node2.Items.Add(node3);
                                }
                            }
                            catch { }
                        }
                    }
                }
                catch (Exception) {
                }
            }
            // showPath(path);  // TreeViewList, falsch, da diese Routine auch von expanded aufgerufen wird
        }  // insertSubDirintoTrees




        // this method must call, if user delete subdirs or insert sub via Strg+C/Strg+V
        public void insertSubDirsIntoTree(TreeViewItem node1, bool clearSubs) {
            TreeViewItem node2, node3;
            if (node1 != null) {
                String path = (String)node1.Tag;
                if (node1.Tag != null && !Directory.Exists(path)) {
                    // Basis.ErrorMsg("Das Verzeichnis ist nicht vorhanden\r\n" + path, "Wechseln in ein Verzeichnis");
                    return;
                }
                bool search = false;
                if (clearSubs && node1.HasItems) {
                    node1.Items.Clear();
                    search = true;
                }
                else {
                    // check, if the node hat Items, BUT node with subdirs has at first only ONE node with "?"
                    // if the node hat Items, don't search again
                    search = !node1.HasItems;
                    if (!search) {
                        // test if item ist "?", ween search
                        node2 = (TreeViewItem)node1.Items[0];
                        if (node2.Header.Equals("?")) {
                            search = true;
                            node1.Items.Clear();
                        }
                    }
                    else {
                    }
                }

                if (search) {
                    try {
                        // search for subdirs
                        String[] dirs = Directory.GetDirectories(path);
                        foreach (String item in dirs) {
                            DirectoryInfo dir = new DirectoryInfo(item);
                            FileAttributes attribs = dir.Attributes;
                            if ((attribs & FileAttributes.Hidden) != FileAttributes.Hidden && (attribs & FileAttributes.System) != FileAttributes.System) {
                                node2 = new TreeViewItem();
                                node2.IsTabStop = false;



                                StackPanel panel = new StackPanel();
                                panel.Orientation = Orientation.Horizontal;

                                TextBlock text4 = new TextBlock();
                                text4.Text = dir.Name;
                                Image bild1 = new Image();
                                BitmapImage bi1 = new BitmapImage();
                                bi1.BeginInit();
                                bi1.UriSource = new Uri("directory.png", UriKind.RelativeOrAbsolute);
                                bi1.EndInit();
                                bild1.Source = bi1;
                                double w = bi1.Width;
                                double h = bi1.Height;
                                panel.Children.Add(bild1);
                                panel.Children.Add(text4);
                                node2.Header = panel;
                                node2.Tag = dir.FullName;
                                node1.Items.Add(node2);
                                try {
                                    String[] dirs2 = Directory.GetDirectories(dir.FullName);
                                    if (dirs2.Length > 0) {
                                        node3 = new TreeViewItem();
                                        node3.IsTabStop = false;
                                        node3.Header = "?";
                                        node3.Tag = null;
                                        node2.Items.Add(node3);
                                    }
                                }
                                catch { }
                            }
                        }
                    }
                    catch (Exception) {
                    }
                }
                // showPath(path);  // TreeViewList, falsch, da diese Routine auch von expanded aufgerufen wird
            }
            else {
                // Title = "node1 == null";
            }
        }  // insertSubDirintoTrees


        private TreeViewItem searchNode(TreeViewItem parent, int index, String[] dirs) {
            // search the actual folder in parent
            String folder = dirs[index];
            // String header = (String) parent.Header;
            String path = "";
            if (_treeNodeFavoritVisible) {
                foreach (TreeViewItem item in parent.Items) {
                    item.IsExpanded = false;
                }
            }
            foreach (TreeViewItem item in parent.Items) {
                path = (String)item.Tag;
                if (path.Equals(folder)) {
                    if (index == (dirs.Length - 1))
                        return item;
                    else {
                        //item.IsExpanded = true;
                        // item.IsSelected = true;
                        if (_treeNodeFavoritVisible) {
                            item.IsExpanded = true;
                        }
                        insertSubDirsIntoTree(item, false);
                        return searchNode(item, index + 1, dirs);
                    }
                }
            }

            return null;
        }


        private void TreeViewItem_Expanded(object sender, RoutedEventArgs e) {
                TreeViewItem node1 = e.OriginalSource as TreeViewItem;
                if (node1 != null) {
                    insertSubDirsIntoTree(node1, false);
                }
        }


        private bool _treeNodeFavoritVisible = false;
        private bool _search = false;

        // single Click
        private void tree_SelectedItemChanged(object sender, RoutedPropertyChangedEventArgs<object> e) {
            tree_click(false);
            e.Handled = true; // abgearbeitet
        }

        private void tree_PreviewMouseDoubleClick(object sender, MouseButtonEventArgs e) {
            tree_click(true);
            e.Handled = true; // abgearbeitet
        }


        // if parent is _mainNodeFavoriten, then search in Node _mainNodeDrives 
        private void tree_click(bool sendToListView) {
            if (_search) return;
            TreeViewItem node1 = (TreeViewItem)tree.SelectedItem;
            if (node1 == null) return;
            String path = (String)node1.Tag;
            if (path == null) {
                //MessageBox.Show("Das Laufwerk ist null", "Wechseln in ein Laufwerk", MessageBoxButton.OK, MessageBoxImage.Error);
                return;
            }
            if (Directory.Exists(path)) {
                if (path.Length <= 3) {
                    DriveInfo drv = new DriveInfo(path);
                    if (!drv.IsReady) {
                        MessageBox.Show("Das Laufwerk ist nicht bereit\r\n" + path, "Wechseln in ein Laufwerk", MessageBoxButton.OK, MessageBoxImage.Error);
                        return;
                    }
                }
                if (sendToListView) {
                    if (_focusfileTable != null) {
                        _focusfileTable.ActualPath = path;
                    }
                }
            }
            insertSubDirsIntoTree(node1, false);
        }


        private void DeleteExecuted(object sender, ExecutedRoutedEventArgs e) {
            deleteFiles(true);
        }
        private void deleteFiles(bool recycleBin) {
            _focusfileTable.DeleteFiles(recycleBin);
        }


        private void CopyExecuted(object sender, ExecutedRoutedEventArgs e) {
            Copy();
        }
        private void Copy() {
            _focusfileTable.Copy();
        }

        private void PasteExecuted(object sender, ExecutedRoutedEventArgs e) {
            Paste();
        }

        private void Paste() {
            _focusfileTable.Paste();
        }



        private void CutExecuted(object sender, ExecutedRoutedEventArgs e) {
            _focusfileTable.Cut();
        }
        private void SelectAllExecuted(object sender, ExecutedRoutedEventArgs e) {
            _focusfileTable.SelectAllItems();
        }
        private void CopyF8Executed(object sender, ExecutedRoutedEventArgs e) {
            _focusfileTable.CopyF8();
        }
        private void RenameF2Executed(object sender, ExecutedRoutedEventArgs e) {
            _focusfileTable.RenameF2();
        }


        private void GoToTopFolderExecuted(object sender, ExecutedRoutedEventArgs e) {
            _focusfileTable.GoToTopFolder();
        }
        private void GoToRootExecuted(object sender, ExecutedRoutedEventArgs e) {
            _focusfileTable.GoToRoot();
        }



        private void NewFolderExecuted(object sender, ExecutedRoutedEventArgs e) {
            NewFolder();
        }
        private void NewFolder() {
            _focusfileTable.NewFolder();
        }

        

        private void LoadAgainExecuted(object sender, ExecutedRoutedEventArgs e) {
            _focusfileTable.LoadAgain();
        }


        private void ShowAllFilesExecuted(object sender, ExecutedRoutedEventArgs e) {
            _focusfileTable.ShowAllFiles();
        }

        private void CompareFoldersExecuted(object sender, ExecutedRoutedEventArgs e) {
            _fileTableLeft.CompareFolders(_fileTableRight.ActualPath, _fileTableRight.Fileliste);
        }


        private void Path2ClipboardExecuted(object sender, ExecutedRoutedEventArgs e) {
            _focusfileTable.Path2Clipboard();
        }


        private void Suche(char abc) {
            _focusfileTable.Suche(abc);
        } // suche


        private void mnExplorer_Click(object sender, RoutedEventArgs e) {
            Process Prog = new Process();
            Prog.StartInfo.FileName = "Explorer.exe";
            Prog.StartInfo.Arguments = "";
            Prog.Start();
        }

        private void mnNotepad_Click(object sender, RoutedEventArgs e) {
            Process Prog = new Process();
            Prog.StartInfo.FileName = "Notepad.exe";
            Prog.StartInfo.Arguments = "";
            Prog.Start();
        }

        private void DosEbeneExecuted(object sender, ExecutedRoutedEventArgs e) {
            _focusfileTable.DosEbene();
        }

        private void mnTaskManager_Click(object sender, RoutedEventArgs e) {
            Process Prog = new Process();
            Prog.StartInfo.FileName = "Taskmgr.exe";
            Prog.StartInfo.Arguments = "";
            Prog.Start();
        }

        private void mnSystemsteuerung_Click(object sender, RoutedEventArgs e) {
            Process Prog = new Process();
            Prog.StartInfo.FileName = "control.exe";
            Prog.StartInfo.Arguments = "";
            Prog.Start();
        }

        private void mnSytemConfig_Click(object sender, RoutedEventArgs e) {
            String msconfig = "";
            if (System.Environment.Is64BitOperatingSystem)
                msconfig = "C:\\Windows\\SysNative\\msconfig.exe";
            else
                msconfig = "c:\\Windows\\System32\\msconfig.exe";
            if (File.Exists(msconfig)) {
                Process Prog = new Process();
                Prog.StartInfo.FileName = msconfig;
                Prog.StartInfo.Arguments = "";
                Prog.Start();
            }
            else {
                Basis.ErrorMsg("Die Datei ist nicht vorhanden\r\n" + msconfig, "Aufrufen eines externen Programms");
            }
        }

        private void mnmsinfo32_Click(object sender, RoutedEventArgs e) {
            Process Prog = new Process();
            Prog.StartInfo.FileName = "msinfo32.exe";
            Prog.StartInfo.Arguments = "";
            Prog.Start();
        }



        private void mnRegedit_Click(object sender, RoutedEventArgs e) {
            Process Prog = new Process();
            Prog.StartInfo.FileName = "Regedit.exe";
            Prog.StartInfo.Arguments = "";
            Prog.Start();
        }

        private void mnCalc_Click(object sender, RoutedEventArgs e) {
            Process Prog = new Process();
            Prog.StartInfo.FileName = "Calc.exe";
            Prog.StartInfo.Arguments = "";
            Prog.Start();
        }

        private void setMask(MenuItem mn) {
            mnMask_stern.IsChecked = false;
            mnMask_txt.IsChecked = false;
            mnMask_ppt.IsChecked = false;
            mnMask_java.IsChecked = false;
            mnMask_doc.IsChecked = false;
            mnMask_xls.IsChecked = false;
            mnMask_pdf.IsChecked = false;
            mnMask_doku.IsChecked = false;
            mnMask_exe.IsChecked = false;
            mnOwnerDefined.IsChecked = false;
            mn.IsChecked = true;
        }

        private void OwnerDefinedMaskExecuted(object sender, ExecutedRoutedEventArgs e) {
            FOwnerDefinedMask dialog = new FOwnerDefinedMask(this, "Selbstdefinierte Maske", "Selbstdefinierte Maske", "Masken, getrennt mit Semikolon");

            dialog.tInput.Text = "*.doc* ; *.xls*";
            dialog.ShowDialog();
            if (dialog.Retcode) {
                _focusfileTable.OwnerDefined = dialog.tInput.Text.Trim();
                _focusfileTable.setMask(Middleware.Masken.ownerdef);
                
                mnOwnerDefined.Header = dialog.tInput.Text.Trim();
                setMask(mnOwnerDefined);
            }
        }

        private void mnMask_txt_Click(object sender, RoutedEventArgs e) {
            _focusfileTable.setMask(Middleware.Masken.txt);
            mnOwnerDefined.Header = "_selbst. def. Maske";
            setMask(mnMask_txt);
        }

        private void mnMask_ppt_Click(object sender, RoutedEventArgs e) {
            _focusfileTable.setMask(Middleware.Masken.ppt);
            mnOwnerDefined.Header = "_selbst. def. Maske";
            setMask(mnMask_ppt);
        }

        private void mnMask_java_Click(object sender, RoutedEventArgs e) {
            _focusfileTable.setMask(Middleware.Masken.java);
            mnOwnerDefined.Header = "_selbst. def. Maske";
            setMask(mnMask_java);
        }

        private void mnMask_doc_Click(object sender, RoutedEventArgs e) {
            _focusfileTable.setMask(Middleware.Masken.doc);
            mnOwnerDefined.Header = "_selbst. def. Maske";
            setMask(mnMask_doc);
        }

        private void mnMask_xls_Click(object sender, RoutedEventArgs e) {
            _focusfileTable.setMask(Middleware.Masken.xls);
            mnOwnerDefined.Header = "_selbst. def. Maske";
            setMask(mnMask_xls);
        }


        private void mnMask_exe_Click(object sender, RoutedEventArgs e) {
            _focusfileTable.setMask(Middleware.Masken.exe);
            mnOwnerDefined.Header = "_selbst. def. Maske";
            setMask(mnMask_exe);
        }

        private void mnMask_stern_Click(object sender, RoutedEventArgs e) {
            _focusfileTable.setMask(Middleware.Masken.Stern);
            mnOwnerDefined.Header = "_selbst. def. Maske";
            setMask(mnMask_stern);
        }

        private void mnMask_pdf_Click(object sender, RoutedEventArgs e) {
            _focusfileTable.setMask(Middleware.Masken.pdf);
            mnOwnerDefined.Header = "_selbst. def. Maske";
            setMask(mnMask_pdf);
        }

        private void mnMask_doku_Click(object sender, RoutedEventArgs e) {
            _focusfileTable.setMask(Middleware.Masken.doku);
            mnOwnerDefined.Header = "_selbst. def. Maske";
            setMask(mnMask_doku);
        }


        private void Window_Closing(object sender, CancelEventArgs e) {
            ColumnDefinitionCollection cols = MainGrid.ColumnDefinitions;
            Basis.WriteRegInt(Konstanten.C_REG_ADR, "Splitterwidth1", (int)cols[0].Width.Value);
            Basis.WriteRegInt(Konstanten.C_REG_ADR, "Splitterwidth2", (int)cols[1].Width.Value);

            Basis.WriteRegInt(Konstanten.C_REG_ADR, "iTop1", (int)this.Top);
            Basis.WriteRegInt(Konstanten.C_REG_ADR, "iLeft1", (int)this.Left);
            Basis.WriteRegInt(Konstanten.C_REG_ADR, "iWidth1", (int)this.Width);
            Basis.WriteRegInt(Konstanten.C_REG_ADR, "iHeight1", (int)this.Height);


            Basis.WriteRegString(Konstanten.C_REG_ADR, "Folder1", _fileTableLeft.ActualPath);
            Basis.WriteRegString(Konstanten.C_REG_ADR, "Folder2", _fileTableRight.ActualPath);
        }


        public UListView getActiveListView(bool left) {
            if (left)
                return _fileTableLeft;
            else
                return _fileTableRight;

    }

    public void setActualMask(Middleware.Masken mask, String ownerdef) {
            switch (mask) {
                case Middleware.Masken.doc: setMask(mnMask_doc); break;
                case Middleware.Masken.doku: setMask(mnMask_doku); break;
                case Middleware.Masken.exe: setMask(mnMask_exe); break;
                case Middleware.Masken.java: setMask(mnMask_java); break;
                case Middleware.Masken.pdf: setMask(mnMask_pdf); break;
                case Middleware.Masken.ppt: setMask(mnMask_ppt); break;
                case Middleware.Masken.Stern: setMask(mnMask_stern); break;
                case Middleware.Masken.txt: setMask(mnMask_txt); break;
                case Middleware.Masken.ownerdef: setMask(mnOwnerDefined);
                    mnOwnerDefined.Header = ownerdef;
                    break;
            }
        }



      
      
        private void Window_SizeChanged(object sender, SizeChangedEventArgs e) {
            //if (!_start) {
            //    ColumnDefinitionCollection cols = MainGrid.ColumnDefinitions;
            //    ColumnDefinition coldef0 = cols[0];
            //    ColumnDefinition coldef1 = cols[1];

            //    double w = this.ActualWidth - tree.ActualWidth - 10;
            //    if (w > 100) {
            //        coldef0.Width = new GridLength(w * 0.5);
            //        coldef1.Width = new GridLength(w * 0.5);
            //    }
            //}
        }


        private void NewInstanzExecuted(object sender, ExecutedRoutedEventArgs e) {
        }



        private void OptionsExecuted(object sender, ExecutedRoutedEventArgs e) {
            FOptions dialog = new FOptions(this);
            dialog.ShowDialog();
            if (dialog.Retcode) {
                setFonts();
                insertFavoriten(_mainNodeFavoriten);
                insertProgs(mainProgs);
                insertLinks(mainLinks);
                for (int i = 0; i < tabcontrol1.Items.Count; i++) {
                    MyTabItem tabitem = (MyTabItem)tabcontrol1.Items[i];
                    UListView fileTable = (UListView)tabitem.Content;
                    fileTable.setContextMenu();
                    fileTable.setFavoriten();
                }
                for (int i = 0; i < tabcontrol2.Items.Count; i++) {
                    MyTabItem tabitem = (MyTabItem)tabcontrol2.Items[i];
                    UListView fileTable = (UListView)tabitem.Content;
                    fileTable.setContextMenu();
                    fileTable.setFavoriten();
                }
                tabcontrol1.FontSize = MyRegistry.ListViewSize;
                tabcontrol2.FontSize = MyRegistry.ListViewSize;
            }
        }

        private void setFonts() {
            mainMenu.FontSize = MyRegistry.MenuSize;
            tree.FontSize = MyRegistry.TreeSize;
            for (int i = 0; i < tabcontrol1.Items.Count; i++) {
                MyTabItem tabitem = (MyTabItem)tabcontrol1.Items[i];
                UListView fileTable = (UListView)tabitem.Content;
                fileTable.FontSize = MyRegistry.ListViewSize;
            }
            for (int i = 0; i < tabcontrol2.Items.Count; i++) {
                MyTabItem tabitem = (MyTabItem)tabcontrol2.Items[i];
                UListView fileTable = (UListView)tabitem.Content;
                fileTable.FontSize = MyRegistry.ListViewSize;
            }           
        }

        
        
        private void ProgInfoExecuted(object sender, ExecutedRoutedEventArgs e) {
            FProgInfo dialog = new FProgInfo(this);
            dialog.ShowDialog();
        }


        private void DeleteTabExecuted(object sender, ExecutedRoutedEventArgs e) {
            if (_activeTabControl.Items.Count > 1) {
                MyTabItem tabPage1 = (MyTabItem)_activeTabControl.Items[_activeTabControl.SelectedIndex];
                _activeTabControl.Items.RemoveAt(_activeTabControl.SelectedIndex);
            }
        }

        private void NewRegisterWithPath(TabControl tabcontrol, TreeViewItem node1) {
            if (node1 != null) {
                String path = (String)node1.Tag;  // Favoriten etc.
                if (path != null) {
                    UListView fileTable;
                    if (path != null) {
                        MyTabItem tabitem1 = new MyTabItem(this, tabcontrol);
                        if (tabcontrol == tabcontrol1) {
                            fileTable = new UListView();
                            fileTable.init(this, this, true, tabcontrol1,  tabitem1, path, new DelegateManager.setActiveRegister(setActiveRegister));
                        }
                        else {
                            fileTable = new UListView();
                            fileTable.init(this, this, false, tabcontrol2, tabitem1, path, new DelegateManager.setActiveRegister(setActiveRegister));
                        }
                        tabitem1.Content = fileTable;
                        tabitem1.IsTabStop = false;
                        tabcontrol.Items.Add(tabitem1);
                        tabitem1.Focus();
                        _focusfileTable = fileTable;
                        _activeTabItem = tabitem1;
                        _focusfileTable = fileTable;

                        _fileTableLeft = fileTable;
                                                                       
                        fileTable.FontSize = MyRegistry.ListViewSize;
                        

                    }
                }
            }
        }

        private void mnNewRegisterLeftWithPath_Click(object sender, RoutedEventArgs e) {
            TreeViewItem node1 = (TreeViewItem)tree.SelectedItem;
            NewRegisterWithPath(tabcontrol1, node1);
        }

        private void mnNewRegisterRightWithPath_Click(object sender, RoutedEventArgs e) {
            TreeViewItem node1 = (TreeViewItem)tree.SelectedItem;
            NewRegisterWithPath(tabcontrol2, node1);
        } 

        private void NewRegisterWithPathExecuted(object sender, ExecutedRoutedEventArgs e) {
            TreeViewItem node1 = (TreeViewItem)tree.SelectedItem;
            NewRegisterWithPath(_activeTabControl, node1);
        }

        private void NewRegisterExecuted(object sender, ExecutedRoutedEventArgs e) {
            if (_activeTabControl != null) {
                String actualPath = "";
                bool left = true;
                if (tabcontrol1 == _activeTabControl) {
                    actualPath = ""; //  listViewFilename1.Text;  imageimage
                    if (actualPath.Length == 0) {
                        actualPath = Basis.ReadRegString(Konstanten.C_REG_ADR, "Folder1");
                        if (!Directory.Exists(actualPath)) actualPath = getFirstFolder();
                    }
                    left = true;
                }
                else {
                    actualPath = ""; //  listViewFilename2.Text;  imageimage
                    if (actualPath.Length == 0) {
                        actualPath = Basis.ReadRegString(Konstanten.C_REG_ADR, "Folder2");
                        if (!Directory.Exists(actualPath)) actualPath = getFirstFolder();
                    }
                    left = false;
                }
                MyTabItem tabitem = new MyTabItem(this, _activeTabControl);
                tabitem.IsTabStop = false;
                UListView fileTable = new UListView();
                fileTable.init(this, this, left, _activeTabControl, tabitem, actualPath, new DelegateManager.setActiveRegister(setActiveRegister));
                tabitem.Content = fileTable;
                _activeTabControl.Items.Add(tabitem);
                tabitem.Focus();
                _focusfileTable = fileTable;

                _activeTabItem = tabitem;
                _focusfileTable = fileTable;
                if (tabcontrol1 == _activeTabControl)
                    _fileTableLeft = fileTable;
                else
                    _fileTableRight = fileTable;

                fileTable.FontSize = MyRegistry.ListViewSize;
            }
        }

        public void DeleteTabExecuted(TabControl tabcontrol, TabItem tabitem) {
            if (tabcontrol.Items.Count > 1) {
                tabcontrol.Items.Remove(tabitem);
                // tabcontrol.Items.RemoveAt(tabcontrol.SelectedIndex);
            }
        }//DeleteTabExecuted

        private void mnShowDriveSpace_Click(object sender, RoutedEventArgs e) {            
            FShowDriveSpace dialog = new FShowDriveSpace(this,"");
            dialog.ShowDialog();
        }

        private void mnShowCountRows_Click(object sender, RoutedEventArgs e) {
            if (_activeTabControl != null) {
                String actualPath = _focusfileTable.ActualPath;
                if (Directory.Exists(actualPath)) {
                    FShowCountRows dialog = new FShowCountRows(this, actualPath);
                    dialog.ShowDialog();
                }
            }
        }

        public void showDriveInfo(String drive) {
            FShowDriveSpace dialog = new FShowDriveSpace(this, drive);
            dialog.ShowDialog();
        }

        private void mnChangeExtention_Click(object sender, RoutedEventArgs e) {
            _focusfileTable.ChangeExtention();
        }


        private void mnRenameWithNumbers_Click(object sender, RoutedEventArgs e) {
            _focusfileTable.RenameWithNumbers();
        }


    }
}
