﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Runtime.InteropServices;  // new


namespace FM6000_Listview {
    public class ThreadCopyDeleteMove {

        SHFILEOPSTRUCT fs;
        bool ok = true;

        public ThreadCopyDeleteMove(string from, String to, FileOperationType fileop, FileOperationFlags flags) {
            try {
                fs = new SHFILEOPSTRUCT {
                    wFunc = fileop,
                    pFrom = from + '\0' + '\0',
                    pTo = to,
                    fFlags = flags
                };
                
            }
            catch (Exception) {
                ok = false;
            }
        }


        public void run() {
            if (ok) {
                SHFileOperation(ref fs);
            }
        }


        [Flags]
        public enum FileOperationFlags : ushort {
            /// <summary>
            /// Do not show a dialog during the process
            /// </summary>
            FOF_SILENT = 0x0004,
            /// <summary>
            /// Do not ask the user to confirm selection
            /// </summary>
            FOF_NOCONFIRMATION = 0x0010,
            /// <summary>
            /// Delete the file to the recycle bin.  (Required flag to send a file to the bin
            /// </summary>
            FOF_ALLOWUNDO = 0x0040,
            /// <summary>
            /// Do not show the names of the files or folders that are being recycled.
            /// </summary>
            FOF_SIMPLEPROGRESS = 0x0100,
            /// <summary>
            /// Surpress errors, if any occur during the process.
            /// </summary>
            FOF_NOERRORUI = 0x0400,
            /// <summary>
            /// Warn if files are too big to fit in the recycle bin and will need
            /// to be deleted completely.
            /// </summary>
            FOF_WANTNUKEWARNING = 0x4000,
        }

        /// <summary>
        /// File Operation Function Type for SHFileOperation
        /// </summary>
        public enum FileOperationType : uint {
            /// <summary>
            /// Move the objects
            /// </summary>
            FO_MOVE = 0x0001,
            /// <summary>
            /// Copy the objects
            /// </summary>
            FO_COPY = 0x0002,
            /// <summary>
            /// Delete (or recycle) the objects
            /// </summary>
            FO_DELETE = 0x0003,
            /// <summary>
            /// Rename the object(s)
            /// </summary>
            FO_RENAME = 0x0004,
        }



        /// <summary>
        /// SHFILEOPSTRUCT for SHFileOperation from COM
        /// </summary>
        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Auto, Pack = 1)]
        public struct SHFILEOPSTRUCT {
            public IntPtr hwnd;
            [MarshalAs(UnmanagedType.U4)]
            public FileOperationType wFunc;
            public string pFrom;
            public string pTo;
            public FileOperationFlags fFlags;
            [MarshalAs(UnmanagedType.Bool)]
            public bool fAnyOperationsAborted;
            public IntPtr hNameMappings;
            public string lpszProgressTitle;
        }







        // Link to the DLL-method
        [DllImport("shell32.dll", CharSet = CharSet.Auto)]
        private static extern int SHFileOperation(ref SHFILEOPSTRUCT FileOp);

        /// <summary>
        /// call the DLL-method 'SHFileOperation'
        /// </summary>
        /// <param name="from">All durs and files with \0</param>
        /// <param name="to">path, in which then files are copied</param>
        /// <param name="flags">fileop: move, cut, copy</param>
        /// <param name="flags">FileOperationFlags to add in addition to FOF_ALLOWUNDO</param>
        /// , FileOperationFlags flags
        public bool FilesAction1(string from, String to, FileOperationType fileop, FileOperationFlags flags) {
            try {
                var fs = new SHFILEOPSTRUCT {
                    wFunc = fileop,
                    pFrom = from + '\0' + '\0',
                    pTo = to,
                    fFlags = flags
                };
                SHFileOperation(ref fs);
                return true;
            }
            catch (Exception) {
                return false;
            }
        }


    }
}
